#pragma once

#define UI_MARGIN 5
#define MIN_FPS 24
#define MAX_FPS 240
#define REPORT_BUFFER_SIZE 128

#include <optional>
#include "glm.hpp"
#include "circular_buffer.hpp"

/// Savable game options
struct options {
    bool show_debug_menu = false;
    bool show_debug_render = false;
    int target_fps = 60;
    int samples = -1;
    bool fullscreen = false;
    ImVec4 clear_color = ImColor(85, 56, 104);
    bool renderer_triplanar = false;
    bool renderer_blend = false;
    bool renderer_fog = true;
    bool renderer_wireframe = false;
    bool culling = true;

    bool show_debug_world = false;

    bool show_debug_contouring = false;
    int contouring_idx = 0;

    bool show_debug_controls = false;

    bool show_overlay = true;
    int overlay_corner = 0;

    bool show_console = false;
    bool console_scrool = true;
};

/// Live state
struct state {
    bool capture_mouse = true;
    //std::optional<std::pair<voxel_pos, Voxel>> look_at = {};

    char console_buffer[256];
};

/// Readonly metrics
struct reports {
    struct main {
        size_t tris_count = 0;
        size_t models_count = 0;
        circular_buffer<float> fps = circular_buffer<float>(REPORT_BUFFER_SIZE, 0);
        circular_buffer<float> update = circular_buffer<float>(REPORT_BUFFER_SIZE, 0);
        circular_buffer<float> render = circular_buffer<float>(REPORT_BUFFER_SIZE, 0);
        circular_buffer<float> swap = circular_buffer<float>(REPORT_BUFFER_SIZE, 0);
        circular_buffer<float> wait = circular_buffer<float>(REPORT_BUFFER_SIZE, 0);
    } main;
    struct world {
        circular_buffer<float> chunk_count = circular_buffer<float>(REPORT_BUFFER_SIZE, 0); // MAYBE: store int
        circular_buffer<float> chunk_load = circular_buffer<float>(REPORT_BUFFER_SIZE, 0);
        circular_buffer<float> chunk_unload = circular_buffer<float>(REPORT_BUFFER_SIZE, 0);
    } world;
};